@@ -1,14 +1,15 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
     SPDX-FileCopyrightText: 2010 Miha Čančula <miha.cancula@gmail.com>
+    SPDX-FileCopyrightText: 2018-2022 Alexander Semke <alexander.semke@web.de>
 */
 
 #include "defaultvariablemodel.h"
-#include <QDebug>
-#include <KLocalizedString>
 #include "extension.h"
 #include "backend.h"
 
+#include <KLocalizedString>
+
 namespace Cantor
 {
 
@@ -17,9 +18,8 @@ class DefaultVariableModelPrivate
 public:
     QList<DefaultVariableModel::Variable> variables;
     QStringList functions;
-
-    Session* session;
-    VariableManagementExtension* extension;
+    Session* session = nullptr;
+    VariableManagementExtension* extension = nullptr;
 };
 
 DefaultVariableModel::DefaultVariableModel(Session* session): QAbstractTableModel(session),
@@ -28,10 +28,7 @@ d_ptr(new DefaultVariableModelPrivate)
     Q_D(DefaultVariableModel);
     d->session = session;
     if (session)
-    {
         d->extension = dynamic_cast<Cantor::VariableManagementExtension*>(session->backend()->extension(QStringLiteral("VariableManagementExtension")));
-    }
-    qDebug() << d->session << d->extension;
 }
 
 DefaultVariableModel::~DefaultVariableModel()
@@ -43,7 +40,12 @@ DefaultVariableModel::~DefaultVariableModel()
 int DefaultVariableModel::columnCount(const QModelIndex& parent) const
 {
     Q_UNUSED(parent);
-    return ColumnCount;
+    Q_D(const DefaultVariableModel);
+    const auto& capabilities = d->session->backend()->capabilities();
+    if (capabilities.testFlag(Cantor::Backend::VariableDimension))
+        return ColumnCount;
+    else
+        return ColumnCount - 1;
 }
 
 int DefaultVariableModel::rowCount(const QModelIndex& parent) const
@@ -65,7 +67,12 @@ QVariant DefaultVariableModel::headerData(int section, Qt::Orientation orientati
         switch(section) {
             case NameColumn:
                 return i18nc("@title:column", "Name");
-
+            case TypeColumn:
+                return i18nc("@title:column", "Type");
+            case SizeColumn:
+                return i18nc("@title:column", "Size [Bytes]");
+            case DimensionColumn:
+                return i18nc("@title:column", "Dimension");
             case ValueColumn:
                 return i18nc("@title:column", "Value");
                 break;
@@ -82,31 +89,34 @@ Qt::ItemFlags DefaultVariableModel::flags(const QModelIndex& index) const
 QVariant DefaultVariableModel::data(const QModelIndex& index, int role) const
 {
     if ((role != Qt::DisplayRole && role != DataRole) || !index.isValid())
-    {
         return QVariant();
-    }
 
     Q_D(const DefaultVariableModel);
+    const auto& variable = d->variables.at(index.row());
     switch (index.column())
     {
         case NameColumn:
-            return QVariant(d->variables[index.row()].name);
+            return QVariant(variable.name);
         case ValueColumn:
-        {
-            const Variable& var = d->variables[index.row()];
-            if (var.value.size() < 1000 || role == DefaultVariableModel::DataRole)
-                return QVariant(var.value);
+            if (variable.value.size() <= 100 || role == DefaultVariableModel::DataRole)
+                return QVariant(variable.value);
             else
-            {
-                if (var.size != 0)
-                    return QVariant(i18n("<%1 bytes>", QString::number(var.size)));
-                else
-                    return QVariant(i18n("<too big variable>"));
-            }
+                return QVariant(variable.value.left(100) + QStringLiteral("..."));
+        case TypeColumn:
+            return QVariant(variable.type);
+        case SizeColumn:
+        {
+            size_t size = variable.size;
+            if (size != 0)
+                return QVariant(qlonglong(size));
+            else
+                return {};
         }
-        default:
-            return QVariant();
+        case DimensionColumn:
+            return QVariant(variable.dimension);
     }
+
+    return {};
 }
 
 bool DefaultVariableModel::setData(const QModelIndex& index, const QVariant& value, int role)
@@ -135,6 +145,7 @@ bool DefaultVariableModel::setData(const QModelIndex& index, const QVariant& val
         emit dataChanged(index, index);
         return true;
     }
+
     return false;
 }
 
@@ -242,18 +253,21 @@ void DefaultVariableModel::setVariables(const QList<DefaultVariableModel::Variab
     {
         bool found = false;
         for (int i = 0; i < size; i++)
-            if(d->variables[i].name == newvar.name)
+        {
+            auto& var = d->variables[i];
+            if(var.name == newvar.name)
             {
-                found=true;
-                if (d->variables[i].value != newvar.value)
+                found = true;
+                if (var.value != newvar.value || var.size != newvar.size || var.type != newvar.type)
                 {
-                    QModelIndex index = createIndex(i, ValueColumn);
-                    d->variables[i].value = newvar.value;
-                    d->variables[i].size = newvar.size;
-                    emit dataChanged(index, index);
+                    var.value = newvar.value;
+                    var.size = newvar.size;
+                    var.type = newvar.type;
+                    emit dataChanged(createIndex(i, NameColumn), createIndex(i, SizeColumn));
                 }
                 break;
             }
+        }
 
         if (!found)
         {
